local ElkFactions = LibStub("AceAddon-3.0"):NewAddon("ElkFactions", "AceConsole-3.0", "AceBucket-3.0")

local LDB = LibStub("LibDataBroker-1.1")
-- local LDBIcon = LibStub("LibDBIcon-1.0")
local LibQTip = LibStub("LibQTip-1.0")

--~ local ElkFactionsLDB

local gender = UnitSex("player")

-- -----
-- HELPER FUNCTIONS
-- -----

local watchedFactionId = nil
local function GetWatchedFactionId()
	local watchedName = GetWatchedFactionInfo()
	if not watchedName then
		-- no faction is currently watched
		watchedFactionId = nil
		return nil
	end
	if watchedFactionId and GetFactionInfo(watchedFactionId) == watchedName then
		-- we have an ID cached and it's still correct
		return watchedFactionId
	end
	
	local factionIndex = 1
	repeat
		if select(12, GetFactionInfo(factionIndex)) then
			watchedFactionId = factionIndex
			return watchedFactionId
		end
		factionIndex = factionIndex + 1
	until factionIndex > 200
end

local function SetWatchedFactionName(factionname)
--	self:Print("SetWatchedFactionName(", factionname, ") called...")
	if not factionname then
		SetWatchedFactionIndex(0)
		return
	end
	local factionIndex = 1
	local lastFactionName
	repeat
		local name = GetFactionInfo(factionIndex)
		if name == lastFactionName then break end
		lastFactionName  = name
		if name == factionname then
			SetWatchedFactionIndex(factionIndex)
			return
		end
		factionIndex = factionIndex + 1
	until factionIndex > 200
end

-- -----
local function do_OnEnter(frame)
	ElkFactions.tooltip = LibQTip:Acquire("ElkFactionsTip")
	ElkFactions.tooltip:SmartAnchorTo(frame)
	ElkFactions:UpdateTooltip()
	ElkFactions.tooltip:Show()
end

local function do_OnLeave()
	if ElkFactions.tooltip then
		LibQTip:Release(ElkFactions.tooltip)
		ElkFactions.tooltip = nil
	end
end

local function do_OnClick(button)
end

function ElkFactions:OnInitialize()
	ElkFactionsLDB = LibStub("LibDataBroker-1.1"):NewDataObject("ElkFactions", {
			type = "data source",
			text = "",
			icon = [[Interface\Icons\Spell_Holy_SealOfSalvation]],
			OnEnter = do_OnEnter,
			OnLeave = do_OnLeave,
			OnClick = do_OnClick,
		})
end

function ElkFactions:OnEnable()
	-- events for faction data changes
	self:RegisterBucketEvent({"UPDATE_FACTION"}, .1, "FactionUpdate")
	
	-- events for zone changes
--~ 	self:RegisterBucketEvent({"ZONE_CHANGED", "ZONE_CHANGED_INDOORS", "ZONE_CHANGED_NEW_AREA", "MINIMAP_ZONE_CHANGED", "PLAYER_ENTERING_WORLD"}, .1, "ZoneUpdate")
end

function ElkFactions:FactionUpdate()
	-- update the LDB text
	self:UpdateText()
	
	-- update the tooltip
	self:UpdateTooltip()
end

local currentZoneCached
function ElkFactions:ZoneUpdate()
	local currentZone = GetRealZoneText().." - "..GetSubZoneText()
	-- if the current zone changed, refresh the automatic zone faction
	if currentZone ~= currentZoneCached then
		self:ApplyAutoZoneFaction()
		currentZoneCached = currentZone
	end
end

function ElkFactions:UpdateText()
	local watchedId = GetWatchedFactionId()
	if watchedId then
		ElkFactionsLDB.text = self:GetFactionText(watchedId, true)
	else
		ElkFactionsLDB.text = ""
	end
end

local FACTION_COLORS_BLIZZARD = setmetatable({}, {__index = function(self, key) local color = FACTION_BAR_COLORS[key]; self[key] = ("|cff%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255); return self[key] end })
local FACTION_COLORS_INC_old = {
	[1] = "|cff8c0000",			-- hated
	[2] = "|cffff0000",			-- hostile
	[3] = "|cffff8c00",			-- unfriendly
	[4] = "|cffbfbfbf",			-- neutral
	[5] = "|cffffffff",			-- friendly
	[6] = "|cff00ff00",			-- honored
	[7] = "|cff3f66e5",			-- reverted
	[8] = "|cff9933cc",			-- exalted
}
local FACTION_COLORS_INC = {
	[1] = "|cffff0000",			-- hated
	[2] = "|cffff8c00",			-- hostile
	[3] = "|cffffff00",			-- unfriendly
	[4] = "|cffffffff",			-- neutral
	[5] = "|cff00ff00",			-- friendly
	[6] = "|cff3f66e5",			-- honored
	[7] = "|cff9933cc",			-- reverted
	[8] = "|cffe6cc80",			-- exalted
}

function ElkFactions:UpdateTooltip()
	if not self.tooltip then return end
	local tooltip = self.tooltip
	tooltip:Clear()
	tooltip:SetColumnLayout(5, "LEFT", "CENTER", "CENTER", "RIGHT", "RIGHT")
	tooltip:AddHeader()
	tooltip:SetCell(1, 1, "|cfffed100Elk|cffffffffFactions", tooltip:GetHeaderFont(), "CENTER", 5)
	local colortest_inc = "  "
	local colortest_inc_old = "  "
	local colortest_blizz = "  "
	for i, c in ipairs(FACTION_COLORS_INC) do
		colortest_inc = colortest_inc..c..GetText("FACTION_STANDING_LABEL"..i, gender).."  "
		colortest_inc_old = colortest_inc_old..FACTION_COLORS_INC_old[i]..GetText("FACTION_STANDING_LABEL"..i, gender).."  "
		colortest_blizz = colortest_blizz..FACTION_COLORS_BLIZZARD[i]..GetText("FACTION_STANDING_LABEL"..i, gender).."  "
	end
	tooltip:AddLine()
	tooltip:SetCell(2, 1, colortest_inc, nil, "CENTER", 5)
	tooltip:AddLine()
	tooltip:SetCell(3, 1, colortest_inc_old, nil, "CENTER", 5)
	tooltip:AddLine()
	tooltip:SetCell(4, 1, colortest_blizz, nil, "CENTER", 5)
	tooltip:AddLine(" ")
	local numFactions = GetNumFactions()
	local header_main = nil
	local header_child = nil
	local indentation
	for id=1, numFactions do
		local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild = GetFactionInfo(id)
		if isHeader then
			if isChild then
				header_child = name
				indentation = isCollapsed and "      |TInterface\\Buttons\\UI-PlusButton-Up:20:20|t" or "      |TInterface\\Buttons\\UI-MinusButton-Up:20:20|t"
			else
				header_child = nil
				header_main = name
				indentation = isCollapsed and "|TInterface\\Buttons\\UI-PlusButton-Up:20:20|t" or "|TInterface\\Buttons\\UI-MinusButton-Up:20:20|t"
			end
		else
			indentation = (header_main and "      " or "")..(header_child and "      " or "")
		end
		local repIs = barValue - barMin
		local repMax = barMax - barMin
		local color_atwar = atWarWith and "|cffff0000" or canToggleAtWar and "|cffffff00" or "|cff00ff00"
		local color_standing_blizzard = FACTION_COLORS_BLIZZARD[standingID]
		local color_standing_inc = FACTION_COLORS_INC[standingID]
		if not isHeader or hasRep then
			tooltip:AddLine(indentation..(isHeader and "|cffffffff" or color_atwar)..name, color_standing_blizzard..GetText("FACTION_STANDING_LABEL"..standingID, gender), ("%s%5d / %5d"):format(color_standing_inc, repIs, repMax), ("%s%.1f%%"):format(color_standing_inc, repIs / repMax * 100), "-")
		else
			tooltip:AddLine(indentation.."|cffffffff"..name)
		end
	end
	tooltip:UpdateScrolling()
end

-- this function will be created from the settings
function ElkFactions:GetFactionText(factionId, useColor)
	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild = GetFactionInfo(factionId)
	local repIs = barValue - barMin
	local repMax = barMax - barMin
	return string.format("%s: %s - %d/%d (%.1f%%) [%+d]", name, GetText("FACTION_STANDING_LABEL"..standingID, gender), repIs, repMax, repIs / repMax * 100, 0)
end

function ElkFactions:GetTooltipLine(factionId)
end

function ElkFactions:ApplyAutoZoneFaction()
	if not self.db.profile.autoZoneFactionEnabled then return end
	
	local currentZone = GetRealZoneText()
	local currentSubZone = GetSubZoneText()
	
	local autoZoneFaction = self.db.profile.AutoZoneFaction[currentZone.." - "..currentSubZone] or self.db.profile.AutoZoneFaction[currentZone]
	-- is there a auto-faction set for the current zone?
	if autoZoneFaction then
		-- if lastFaction is set, last zone had autofaction, too
		-- if not we have to store the current watched faction
		if self.db.char.lastFaction == nil then
			self.db.char.lastFaction = GetWatchedFactionInfo() or false
		end
		self.SettingAutoFaction = true
		SetWatchedFactionName(autoZoneFaction)
	-- no auto-faction set so restore last faction if set
	elseif self.db.char.lastFaction ~= nil then
		self.SettingAutoFaction = true
		SetWatchedFactionName(self.db.char.lastFaction)
		self.db.char.lastFaction = nil
	end
end